# Formula team management

## About Project

- **Name**: Formula One Team

- **Technologies**: Java 17, Spring, Maven, Tomcat

- **Developers**:

    - Alžbeta Hajná [xhajna](https://gitlab.fi.muni.cz/xhajna)
    - Jitka Viceníková [xvicenik](https://gitlab.fi.muni.cz/xvicenik)
    - Andrej Žabka [xzabka](https://gitlab.fi.muni.cz/xzabka)
    - Michal Badin [xbadin](https://gitlab.fi.muni.cz/xbadin)

- **Assigment**:
    - A Formula 1 team is in need to manage its participation to the world championship. The team has two cars
      participating to each race, each one driven by one driver, with other test drivers available to develop the car
      further. The manager of the team can set the driver for each car and manage a list of other (test) drivers that
      are under contract and he can also set as main drivers if unhappy about results. Each driver has a name, surname,
      nationality, and set of characteristics (e.g. driving on the wet, aggressiveness, etc..) that the manager can
      visualize. The car is composed of several components (engine, suspensions), that the different departments of the
      team (e.g. engine, aerodynamics) evolve during time. The manager can pick from a list of components and assemble
      the cars that will participate to the next race. Engineers from other departments can login to the system and put
      newer components available. The manager is notified that those components are available and that they can be used
      to improve the car.

- **Modules**:
  <br>Our application is divided into 4 separate spring boot modules, as shown on the class diagram

1) Core module:
   <br>Core module contains most of the functionality of the project assignment. It communicates with the other modules
   via their REST api's.
2) Notifications module:
   <br>Notifications module sends e-mail (or other) notifications to team members.
3) Visualisation module:
   <br>Visualization module provides a visualization of given JSON data in readable HTML (or other) format. Team manager
   will be able to visualize current state of both cars.
4) Applications module:
   <br>Applications module allows people to send an application to this team. Manager can view these applications and
   hire new drivers for his team.

- **Diagrams**:

![IMAGE_DESCRIPTION](diagrams/class_diagram.png)
![IMAGE_DESCRIPTION](diagrams/use_case_diagram.png)

# Build and run the app

Build the app (all modules) by running

```bash
mvn clean install
```

in the project's parent directory, or build separate modules by running the same command inside the module's parent
directory. For example, to build notifications module:

```bash
cd notification
mvn clean install
```

Tests are run automatically during the build, but if you want to run them again, use:

```bash
mvn test
```

again, either in the parent directory to test all modules, or inside separate modules to only test one at a time.

To run a module, just run:

```bash
cd core
mvn
```

or alternatively:

```bash
cd core
mvn spring-boot:run
```

To see the running REST api's in action, use CURL or swagger UI:
http://localhost:8080/swagger-ui/index.html

Just note that each module runs on a different port by default, so care where you send your requests.<br>
**Default ports:**

- core module: 8080
- notifications module: 8083
- visualization module: 8082
- applications module: 8081

These ports can be overridden either in the application.properties file of each module, or by specifying the port
manually when running the "mvn" command to run the module.

## Build and run the app with Docker

For purpose of this build and run the installation of Docker/Podman
is required, steps to do so are not provided here.

*Note: If you are a win user using Docker Desktop, don't forget to run Docker Desktop first.*

### Building Docker images

To build images you will need `alpine` image
you can get it by running

```bash
docker pull registry.hub.docker.com/library/eclipse-temurin:17-jdk-alpine
```

After successful pull, run

```bash
mvn clean install
```

in root directory to build `jar` file for every module.

*Note: You can use flag `-Dskiptests` to save some time.*

For every module listed above you need to run:

```bash
cd <module>
docker build -t pa165/formula-team-management/<module>-image .
```

### Running Docker images

To run specific docker image use command

```bash
docker run -p 8080:8080 pa165/formula-team-management/<module>-image
```

To avoid confusion the best way is to be consistent with ports listed above,
e.g. for running `application` image use:

```bash
docker run -p 8081:8081 pa165/formula-team-management/application-image
```

### Docker compose

To run all modules at once in one container, you can run in root directory

```bash
docker-compose up
```

*Note: in case you want to also rebuild images, use flag `--build`*
*Note: use flag `-d` for detached mode*

To stop container run

```bash
docker-compose down
```

## Collecting Metrics

To collect the metrics in an automated way we use Prometheus.
First, make sure that every service you want to observe is running on predefined port.
Then run Prometheus in docker with respect to location of `prometheus.yml` file:

```bash
docker run --rm --name prometheus -p 9090:9090 -v ${PWD}/prometheus.yml:/etc/prometheus/prometheus.yml prom/prometheus:v2.43.0 --config.file=/etc/prometheus/prometheus.yml
```
